<!DOCTYPE html>
<html>
<head lang="en">
	<meta charset="UTF-8" />
	<meta http-equiv="x-ua-compatible" content="ie=edge,chrome=1" />
	<meta name="viewport" content="user-scalable=0,initial-scale=1.0,minimal-ui" />
	<title>Pipeline Stats</title>
	<link href="https://maxcdn.bootstrapcdn.com/font-awesome/4.2.0/css/font-awesome.min.css" rel="stylesheet">
	<link href="/style.css" rel="stylesheet">
	<script src="https://code.jquery.com/jquery-1.11.2.min.js"></script>
	<script src="https://fb.me/react-0.14.6.min.js"></script>
	<script src="https://fb.me/react-dom-0.14.6.min.js"></script>
	<script src="https://cdnjs.cloudflare.com/ajax/libs/babel-core/5.8.23/browser.min.js"></script>
</head>
<body>
	<div id="pulse"></div>
	<div id="disconnected-message"><div class="disconnected-inner">DISCONNECTED</div></div>
	<div id="container"></div>
	<script type="text/javascript">
		var PipelineSettings = {
			'apiUrl': '/api/status/',
			'detailUrlPrefix': '/detail/',
			'remainingSecondsWarning': 5 * 60,
			'remainingSecondsDanger': 10 * 60
		}
	</script>
	<script type="text/javascript" src="/app.js"></script>
</body>
</html>